/*
** Module   : BrowseConfig.java
** Abstract : browse widget attribute container.
**
** Copyright (c) 2005-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 SIY 20050909   @22824 Created initial version.
** 002 NVS 20051116   @23346 Constructor access changed to public. 
** 003 SIY 20051130   @23588 Added get/setQuery(). 
** 004 SIY 20060209   @24490 Major refactoring.
** 005 SIY 20060315   @25123 Changes for complete BROWSE implementation.
** 006 SIY 20060329   @25277 Improved encapsulation, added BOX attribute,
**                           added WIDTH attribute.
** 007 SIY 20060406   @25401 Added centered attribute.
** 008 GES 20060712   @27995 Added help attribute.
** 009 GES 20060803   @28392 Made externalizable for better performance.
**                           Many code standard cleanups.
** 010 SIY 20070319   @32460 Avoid setting DOWN if size is specified 
**                           in different way.
** 011 NVS 20070426   @33248 Removed checks of SENSITIVE attribute as
**                           redundant.
** 012 ECF 20080208   @37033 Added getSelectedRowIndex(). Gets the row
**                           index of a selected row in the browse widget.
** 013 SIY 20100619          Switching to own implementation of character UI.
** 014 SIY 20100804          Changes in imports, some refactorings.
** 015 SIY 20110112          Changes related to new color handling scheme.
** 016 SIY 20111101          Moved from CHUI package. Minor 
**                           cleanups/refactorings in preparation to GUI client.
** 017 SIY 20111202          Updated imports.
** 018 GES 20140627          Changed color attribute method names to match naming standards.
** 019 CA  20140711          Enhanced font support, for GUI clients.
** 020 HC  20140911          GUI coordinate support - data types refactoring.
** 021 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 022 CA  20141104          Removed width - it collides with BaseConfig.widthChars.
** 023 SVL 20141102          Added some attributes.
*/

package com.goldencode.p2j.ui;

import java.io.*;

import com.goldencode.p2j.ui.client.*;

/**
 * This class holds all necessary attributes for the Browse widget.
 */
public class BrowseConfig
extends BaseConfig
{
   /** NUM-ITERATIONS attribute variable. */
   public int numIterations = 0;            

   /** SEPARATORS attribute variable. */
   public boolean separators = false;      

   /** 
    * ALLOW-COLUMN-SEARCHING attribute variable.  
    * <p>
    * ISSUE: not implemented.
    */
   public boolean allowColumnSearching = false;

   /** 
    * COLUMN-SCROLLING attribute variable.
    * <p>
    * ISSUE: non-default behavior is not implemented.
    */
   public boolean columnScrolling = true;             

   /** DOWN attribute variable */
   public int down = 10;                                

   /** LABELS attribute variable. */
   public boolean labels = true;            

   /** 
    * MAX-DATA-GUESS attribute variable. 
    * <p>
    * This is ignored, caching does not involve it.
    */ 
   public int maxDataGuess = 100;

   /** 
    * NUM-LOCKED-COLUMNS attribute variable. 
    * <p>
    * ISSUE: not implemented.
    */
   public int numLockedColumns = 0;                

   /** NUM-VISIBLE-COLUMNS attribute variable. */
   public int numVisibleColumns = 1;        

   /** READ-ONLY attribute variable. */
   public boolean readOnly = false;        

   /** REFRESHABLE attribute variable. */
   public boolean refreshable = true;      

   /** 
    * ROW-HEIGHT-CHARS attribute variable. 
    * <p>
    * ISSUE: non-default values are not implemented.
    */
   public double rowHeightChars = 1;      

   /** ROW-MARKERS attribute variable. */
   public boolean rowMarkers = true;       

   /** 
    * SCROLLBAR-VERTICAL attribute variable. 
    * <p>
    * ISSUE: not implemented.
    */
   public boolean scrollbarVertical = true;

   /** HELP attribute variable. */
   public String help = null;             

   /** Support multiple selection. */
   public boolean multiple = false;             

   /** ID of the query associated with Browse instance. */
   public int queryId = -1;

   /** BOX attribute. */
   public boolean box = true;

   // TODO: not clear if it has any meaning or used at all
   /** OVERLAY attribute. */
   public boolean overlay = false;

   /** CENTERED attribute. */
   public boolean centered = false;

   /** NO-ASSIGN attribute. */
   public boolean noAssign = false;

   /** FIT-LAST-COLUMN attribute. */
   public boolean fitLastColumn = false;

   /** NO-UNDERLINE attribute. */
   public boolean noUnderline = false;

   /** SCROLLABLE attribute. */
   public boolean scrollable = false;

   /** COLUMN-RESIZABLE attribute. */
   public boolean columnResizable = false;

   /** COLUMN-MOVABLE attribute. */
   public boolean columnMovable = false;

   /** CREATE-ON-ADD attribute. */
   public boolean createOnAdd = false; //TODO check if the default value is correct

   /**
    * Default constructor (only used in de-serialization).
    */
   public BrowseConfig()
   {
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected BrowseConfig(int id)
   {
      super(id);
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected BrowseConfig(WidgetId id)
   {
      super(id);
   }

   /**
    * Setting the HEIGHT-CHARS Progress attribute.
    * 
    * @param   heightChars
    *          The integer value for the new HEIGHT-CHARS value.
    */
   @Override
   public void setHeightChars(double heightChars)
   {
      this.heightChars = Coordinate.round(heightChars); // direct access
   }
   
   /**
    * Set the WIDTH-CHARS Progress attribute.
    * 
    * @param   widthChars
    *          The integer value for the new WIDTH-CHARS value.
    */
   @Override
   public void setWidthChars(double widthChars)
   {
      this.widthChars = Coordinate.round(widthChars); // direct access
   }
   
   /**
    * Synchronize the configuration with the widget state.
    */
   @Override
   public void syncWithWidget()
   {
      super.syncWithWidget();

      // the rowHeightChars is expected to already be rounded
      
      if (rowHeightChars != 1)
      {
         throw new RuntimeException("ROW-HEIGTH-CHARS other than 1 is not supported");
      }

      if (id == null || !onClientSide())
      {
         return;
      }

      Browse<?> instance = (Browse<?>) tc.getWidget(id);
      
      if (instance == null)
      {
         return;
      }

      if (heightChars == 0 && widthChars == 0)
      {
         instance.setDown(down * (int) rowHeightChars);
      }

      instance.setSelectionMultiple(multiple);
      instance.setReadOnly(readOnly);

      if (queryId != -1)
      {
         instance.setQuery(queryId);
      }
   }

   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      BrowseConfig cfg = (BrowseConfig) config;
      
      numIterations = cfg.numIterations;
      separators = cfg.separators;
      allowColumnSearching = cfg.allowColumnSearching;
      columnScrolling = cfg.columnScrolling;
      down = cfg.down;
      labels = cfg.labels;
      maxDataGuess = cfg.maxDataGuess;
      numLockedColumns = cfg.numLockedColumns;
      numVisibleColumns = cfg.numVisibleColumns;
      readOnly = cfg.readOnly;
      refreshable = cfg.refreshable;
      rowHeightChars = cfg.rowHeightChars;
      rowMarkers = cfg.rowMarkers;
      scrollbarVertical = cfg.scrollbarVertical;
      help = cfg.help;
      multiple = cfg.multiple;
      queryId = cfg.queryId;
      box = cfg.box;
      overlay = cfg.overlay;
      centered = cfg.centered;
      noAssign = cfg.noAssign;
      fitLastColumn = cfg.fitLastColumn;
      noUnderline = cfg.noUnderline;
      scrollable = cfg.scrollable;
      columnResizable = cfg.columnResizable;
      columnMovable = cfg.columnMovable;
      createOnAdd = cfg.createOnAdd;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      numIterations        = in.readInt();
      down                 = in.readInt();
      numLockedColumns     = in.readInt();
      rowHeightChars       = in.readDouble();
      queryId              = in.readInt();
      separators           = in.readBoolean();
      allowColumnSearching = in.readBoolean();
      columnScrolling      = in.readBoolean();
      labels               = in.readBoolean();
      readOnly             = in.readBoolean();
      refreshable          = in.readBoolean();
      rowMarkers           = in.readBoolean();
      scrollbarVertical    = in.readBoolean();
      multiple             = in.readBoolean();
      box                  = in.readBoolean();
      overlay              = in.readBoolean();
      centered             = in.readBoolean();
      help                 = (String) in.readObject();
      maxDataGuess         = in.readInt();
      numVisibleColumns    = in.readInt();
      noAssign             = in.readBoolean();
      fitLastColumn        = in.readBoolean();
      noUnderline          = in.readBoolean();
      scrollable           = in.readBoolean();
      columnResizable      = in.readBoolean();
      columnMovable        = in.readBoolean();
      createOnAdd          = in.readBoolean();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeInt(numIterations);
      out.writeInt(down);
      out.writeInt(numLockedColumns);
      out.writeDouble(rowHeightChars);
      out.writeInt(queryId);
      out.writeBoolean(separators);
      out.writeBoolean(allowColumnSearching);
      out.writeBoolean(columnScrolling);
      out.writeBoolean(labels);
      out.writeBoolean(readOnly);
      out.writeBoolean(refreshable);
      out.writeBoolean(rowMarkers);
      out.writeBoolean(scrollbarVertical);
      out.writeBoolean(multiple);
      out.writeBoolean(box);
      out.writeBoolean(overlay);
      out.writeBoolean(centered);
      out.writeObject(help);
      out.writeInt(maxDataGuess);
      out.writeInt(numVisibleColumns);
      out.writeBoolean(noAssign);
      out.writeBoolean(fitLastColumn);
      out.writeBoolean(noUnderline);
      out.writeBoolean(scrollable);
      out.writeBoolean(columnResizable);
      out.writeBoolean(columnMovable);
      out.writeBoolean(createOnAdd);
   }
}
