/*
** Module   : EditorWidget.java
** Abstract : server side Editor widget implementation
**
** Copyright (c) 2006-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20060427   @26084 Server side part of the EDITOR widget.
** 002 GES 20060803   @28373 Cleanup and refactoring.
** 003 GES 20080320   @37585 Added TYPE attribute.
** 004 SIY 20111101          Updated imports.
** 005 CA  20130129          Added setLarge stub, for EDITOR LARGE.
** 006 CA  20130221          Refactored usage of TYPE attribute.
** 007 CA  20130222          Added NPE protection for cases when a widget is used before is 
**                           attached to a frame (after a CREATE WIDGET statement is used).
** 008 CA  20130927          Added LegacyResource annotation, to determine resource type.
** 009 CA  20131115          Changes related to properly instantiate dynamic resources.
** 010 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 011 SVL 20141102          Make this class implement ScrollbarVerticalElement.
*/

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Server side implementation of the EDITOR widget.
 */
@LegacyResource(resource = LegacyResource.EDITOR)
public class EditorWidget
extends ControlTextWidget<EditorConfig>
implements ScrollbarVerticalElement
{
   /**
    * Default constructor.
    */
   public EditorWidget()
   {
      this(false);
   }
   
   /**
    * Default constructor.
    * 
    * @param   dynamic
    *          Flag indicating if this is a static or dynamic resource.
    */
   public EditorWidget(boolean dynamic)
   {
      super(dynamic, new EditorConfig());
   }

   /**
    * Mark this EDITOR widget as EDITOR LARGE, which can be used to display 
    * longchar value.
    * <p> 
    * This is a stub, it needs to be implemented.
    * 
    * @param    large
    *           When <code>true</code>, this is a EDITOR LARGE widget.
    */
   public void setLarge(boolean large)
   {
      // TODO: implement me
   }
   
   /**
    * Set value of the INNER-LINES attribute.
    * 
    * @param   lines 
    *          The new value for the attribute.
    */
   public void setInnerLines(int lines)
   {
      config.innerLines = lines;
      pushScreenDefinition();
   }
   
   /**
    * Set value of the INNER-LINES attribute.
    * 
    * @param   lines 
    *          The new value for the attribute.
    */
   public void setInnerLines(NumberType lines)
   {
      if (lines.isUnknown())
         return;
      
      setInnerLines(lines.intValue());
   }
   
   /**
    * Set value of the INNER-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setInnerChars(int chars)
   {
      config.innerChars = chars;
      pushScreenDefinition();
   }
   
   /**
    * Set value of the INNER-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setInnerChars(NumberType chars)
   {
      if (chars.isUnknown())
         return;
      
      setInnerChars(chars.intValue());
   }

   /**
    * Set value of the MAX-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setMaxChars(int chars)
   {
      config.maxChars = chars;
      pushScreenDefinition();
   }

   /**
    * Set value of the MAX-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setMaxChars(NumberType chars)
   {
      if (chars.isUnknown())
         return;
      
      setMaxChars(chars.intValue());
   }
   
   /**
    * Set value of the BUFFER-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setBufferChars(int chars)
   {
      config.bufferChars = chars;
      pushScreenDefinition();
   }
   
   /**
    * Set value of the BUFFER-CHARS attribute.
    * 
    * @param   chars 
    *          The new value for the attribute.
    */
   public void setBufferChars(NumberType chars)
   {
      if (chars.isUnknown())
         return;
      
      setBufferChars(chars.intValue());
   }
   
   /**
    * Set value of the BUFFER-LINES attribute.
    * 
    * @param   lines
    *          The new value for the attribute.
    */
   public void setBufferLines(int lines)
   {
      config.bufferLines = lines;
      pushScreenDefinition();
   }
   
   /**
    * Set value of the BUFFER-LINES attribute.
    * 
    * @param   lines
    *          The new value for the attribute.
    */
   public void setBufferLines(NumberType lines)
   {
      if (lines.isUnknown())
         return;
      
      setBufferLines(lines.intValue());
   }
   
   /**
    * Set value of the BOX attribute as opposite to passed value.
    * 
    * @param   box
    *          The new value for the attribute.
    */
   public void setNoBox(boolean box)
   {
      config.box = !box;
      pushScreenDefinition();
   }

   /**
    * Set value of the BOX attribute as opposite to passed value.
    * 
    * @param   box
    *          The new value for the attribute.
    */
   public void setNoBox(logical box)
   {
      if (box.isUnknown())
         return;
      
      setNoBox(box.booleanValue());
   }
   
   /**
    * Set value of the WORD-WRAP attribute as opposite to passed value.
    * 
    * @param   wordwrap
    *          The new value for the attribute.
    */
   public void setNoWordWrap(boolean wordwrap)
   {
      config.wordWrap = !wordwrap;
      pushScreenDefinition();
   }

   /**
    * Set value of the WORD-WRAP attribute as opposite to passed value.
    * 
    * @param   wordwrap
    *          The new value for the attribute.
    */
   public void setNoWordWrap(logical wordwrap)
   {
      if (wordwrap.isUnknown())
         return;
      
      setNoWordWrap(wordwrap.booleanValue());
   }
   
   /**
    * Set value of the SCROLLBAR-VERTICAL attribute.
    * 
    * @param   vertical
    *          The new value for the attribute.
    */
   public void setScrollbarVertical(boolean vertical)
   {
      config.scrollbarVertical = vertical;
      pushScreenDefinition();
   }

   /**
    * Set value of the SCROLLBAR-VERTICAL attribute.
    * 
    * @param   vertical
    *          The new value for the attribute.
    */
   public void setScrollbarVertical(logical vertical)
   {
      if (vertical.isUnknown())
         return;
      
      setScrollbarVertical(vertical.booleanValue());
   }

   /**
    * Get value of the SCROLLBAR-VERTICAL attribute.
    *
    * @return  value of the attribute.
    */
   public logical isScrollbarVertical()
   {
      return new logical(config.scrollbarVertical);
   }
   
   /**
    * Set value of the SCROLLBAR-HORIZONTAL attribute.
    * 
    * @param   horizontal
    *          The new value for the attribute.
    */
   public void setScrollbarHorizontal(boolean horizontal)
   {
      config.scrollbarHorizontal = horizontal;
      pushScreenDefinition();
   }

   /**
    * Set value of the SCROLLBAR-HORIZONTAL attribute.
    * 
    * @param   horizontal
    *          The new value for the attribute.
    */
   public void setScrollbarHorizontal(logical horizontal)
   {
      if (horizontal.isUnknown())
         return;
      
      setScrollbarHorizontal(horizontal.booleanValue());
   }
}
